{- Derived parser combinators
 -
 - Author(s): Doaitse Swierstra
 - Version 1.0, May 14, 1998: first appearance on the software web site.
 -}

module UU_Parsing_ext where

import UU_Parsing

infixl 4  <$>, <$->, <*->, <-*>, <**>, <??>
infixl 2 `opt`

opt    :: Symbol s => Parser s a -> a -> Parser s a
(<$>)  :: Symbol s => (b->a) -> Parser s b -> Parser s a
(<$->) :: Symbol s => a -> Parser s b -> Parser s a
(<*->) :: Symbol s => Parser s a -> Parser s b -> Parser s a
(<-*>) :: Symbol s => Parser s a -> Parser s b -> Parser s b
(<**>) :: Symbol s => Parser s b      -> Parser s (b->a) -> Parser s a
(<??>) :: Symbol s => Parser s b      -> Parser s (b->b) -> Parser s b

--pMany   :: Symbol s => Parser s a -> Parser s [a]
--pSome   :: Symbol s => Parser s a -> Parser s [a]
--pChainr :: Symbol s => Parser s (a -> a -> a) -> Parser s a ->  Parser s a
--pChainl :: Symbol s => Parser s (a -> a -> a) -> Parser s a -> Parser s a

-- definitions of derived parser combinators

p `opt` v = p <|> succeed v
f <$> p   = succeed f <*> p
f <$-> p  = const f <$> p
p <*-> q  = (\ x _ -> x) <$> p <*> q
p <-*> q  = (\ _ x -> x) <$> p <*> q
p <**> q  = (\ x f -> f x) <$> p <*> q
p <??> q  = (\ x f -> f x) <$> p <*> (q `opt` id)

pFoldr         alg@(op,e)     p = pfm where pfm = op <$> p <*> pfm `opt` e
pFoldrSep      alg@(op,e) sep p = op <$> p <*> pFoldr alg (sep <-*> p) 
pFoldrPrefixed alg@(op,e) c   p = pFoldr alg (c <-*> p)
 
pList p           = pManyp where pManyp = (:) <$> p <*> pManyp `opt` []
pListSep      s p = pFoldrSep ((:),[]) s p
pListPrefixed c p = pFoldrPrefixed ((:),[]) c p

pSome p = pSomep where pSomep = (:) <$> p <*> pList p
pChainr op x =  r where r = x <**> (flip <$> op <*> r `opt` id)
pChainl op x = f <$> x <*> pList (flip <$> op <*> x) 
               where
                 f x [] = x
                 f x (func:rest) = f (func x) rest

pPacked l x r
  =   l <-*>  x <*->  r





